@@ -52,6 +52,7 @@
 #include <alsa/asoundlib.h>
 
 #include "gstalsasrc.h"
+#include "gstalsadeviceprobe.h"
 
 #include <gst/gst-i18n-plugin.h>
 
@@ -72,12 +73,14 @@ enum
   PROP_DEVICE_NAME,
 };
 
-GST_BOILERPLATE_WITH_INTERFACE (GstAlsaSrc, gst_alsasrc, GstAudioSrc,
-    GST_TYPE_AUDIO_SRC, GstMixer, GST_TYPE_MIXER, gst_alsasrc_mixer);
+static void gst_alsasrc_init_interfaces (GType type);
+
+GST_BOILERPLATE_FULL (GstAlsaSrc, gst_alsasrc, GstAudioSrc,
+    GST_TYPE_AUDIO_SRC, gst_alsasrc_init_interfaces);
 
 GST_IMPLEMENT_ALSA_MIXER_METHODS (GstAlsaSrc, gst_alsasrc_mixer);
 
-static void gst_alsasrc_dispose (GObject * object);
+static void gst_alsasrc_finalize (GObject * object);
 static void gst_alsasrc_set_property (GObject * object,
     guint prop_id, const GValue * value, GParamSpec * pspec);
 static void gst_alsasrc_get_property (GObject * object,
@@ -111,22 +114,81 @@ static GstStaticPadTemplate alsasrc_src_factory =
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("audio/x-raw-int, "
+        "endianness = (int) { " ALSA_SRC_FACTORY_ENDIANNESS " }, "
+        "signed = (boolean) { TRUE, FALSE }, "
+        "width = (int) 32, "
+        "depth = (int) 32, "
+        "rate = (int) [ 1, MAX ], " "channels = (int) [ 1, MAX ]; "
+        "audio/x-raw-int, "
+        "endianness = (int) { " ALSA_SRC_FACTORY_ENDIANNESS " }, "
+        "signed = (boolean) { TRUE, FALSE }, "
+        "width = (int) 32, "
+        "depth = (int) 24, "
+        "rate = (int) [ 1, MAX ], " "channels = (int) [ 1, MAX ]; "
+        "audio/x-raw-int, "
+        "endianness = (int) { " ALSA_SRC_FACTORY_ENDIANNESS " }, "
+        "signed = (boolean) { TRUE, FALSE }, "
+        "width = (int) 24, "
+        "depth = (int) 24, "
+        "rate = (int) [ 1, MAX ], " "channels = (int) [ 1, MAX ]; "
+        "audio/x-raw-int, "
         "endianness = (int) { " ALSA_SRC_FACTORY_ENDIANNESS " }, "
         "signed = (boolean) { TRUE, FALSE }, "
         "width = (int) 16, "
         "depth = (int) 16, "
-        "rate = (int) [ 1, MAX ], " "channels = (int) [ 1, 2 ]; "
+        "rate = (int) [ 1, MAX ], " "channels = (int) [ 1, MAX ]; "
         "audio/x-raw-int, "
         "signed = (boolean) { TRUE, FALSE }, "
         "width = (int) 8, "
         "depth = (int) 8, "
-        "rate = (int) [ 1, MAX ], " "channels = (int) [ 1, 2 ]")
+        "rate = (int) [ 1, MAX ], " "channels = (int) [ 1, MAX ]")
     );
 
 static void
-gst_alsasrc_dispose (GObject * object)
+gst_alsasrc_finalize (GObject * object)
 {
-  G_OBJECT_CLASS (parent_class)->dispose (object);
+  GstAlsaSrc *src = GST_ALSA_SRC (object);
+
+  g_free (src->device);
+  g_mutex_free (src->alsa_lock);
+
+  G_OBJECT_CLASS (parent_class)->finalize (object);
+}
+
+static gboolean
+gst_alsasrc_interface_supported (GstAlsaSrc * this, GType interface_type)
+{
+  /* only support this one interface (wrapped by GstImplementsInterface) */
+  g_assert (interface_type == GST_TYPE_MIXER);
+
+  return gst_alsasrc_mixer_supported (this, interface_type);
+}
+
+static void
+gst_implements_interface_init (GstImplementsInterfaceClass * klass)
+{
+  klass->supported = (gpointer) gst_alsasrc_interface_supported;
+}
+
+static void
+gst_alsasrc_init_interfaces (GType type)
+{
+  static const GInterfaceInfo implements_iface_info = {
+    (GInterfaceInitFunc) gst_implements_interface_init,
+    NULL,
+    NULL,
+  };
+  static const GInterfaceInfo mixer_iface_info = {
+    (GInterfaceInitFunc) gst_alsasrc_mixer_interface_init,
+    NULL,
+    NULL,
+  };
+
+  g_type_add_interface_static (type, GST_TYPE_IMPLEMENTS_INTERFACE,
+      &implements_iface_info);
+  g_type_add_interface_static (type, GST_TYPE_MIXER, &mixer_iface_info);
+
+  gst_alsa_type_add_device_property_probe_interface (type);
 }
 
 static void
@@ -155,7 +217,7 @@ gst_alsasrc_class_init (GstAlsaSrcClass * klass)
   gstbaseaudiosrc_class = (GstBaseAudioSrcClass *) klass;
   gstaudiosrc_class = (GstAudioSrcClass *) klass;
 
-  gobject_class->dispose = GST_DEBUG_FUNCPTR (gst_alsasrc_dispose);
+  gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_alsasrc_finalize);
   gobject_class->get_property = GST_DEBUG_FUNCPTR (gst_alsasrc_get_property);
   gobject_class->set_property = GST_DEBUG_FUNCPTR (gst_alsasrc_set_property);
 
@@ -190,9 +252,11 @@ gst_alsasrc_set_property (GObject * object, guint prop_id,
 
   switch (prop_id) {
     case PROP_DEVICE:
-      if (src->device)
-        g_free (src->device);
-      src->device = g_strdup (g_value_get_string (value));
+      g_free (src->device);
+      src->device = g_value_dup_string (value);
+      if (src->device == NULL) {
+        src->device = g_strdup (DEFAULT_PROP_DEVICE);
+      }
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -213,16 +277,9 @@ gst_alsasrc_get_property (GObject * object, guint prop_id,
       g_value_set_string (value, src->device);
       break;
     case PROP_DEVICE_NAME:
-      if (src->handle) {
-        snd_pcm_info_t *info;
-
-        snd_pcm_info_malloc (&info);
-        snd_pcm_info (src->handle, info);
-        g_value_set_string (value, snd_pcm_info_get_name (info));
-        snd_pcm_info_free (info);
-      } else {
-        g_value_set_string (value, NULL);
-      }
+      g_value_take_string (value,
+          gst_alsa_find_device_name (GST_OBJECT_CAST (src),
+              src->device, src->handle, SND_PCM_STREAM_CAPTURE));
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -236,12 +293,9 @@ gst_alsasrc_init (GstAlsaSrc * alsasrc, GstAlsaSrcClass * g_class)
   GST_DEBUG_OBJECT (alsasrc, "initializing");
 
   alsasrc->device = g_strdup (DEFAULT_PROP_DEVICE);
-}
+  alsasrc->cached_caps = NULL;
 
-static GstCaps *
-gst_alsasrc_getcaps (GstBaseSrc * bsrc)
-{
-  return NULL;
+  alsasrc->alsa_lock = g_mutex_new ();
 }
 
 #define CHECK(call, error) \
@@ -250,6 +304,43 @@ if ((err = call) < 0)           \
   goto error;                   \
 } G_STMT_END;
 
+
+static GstCaps *
+gst_alsasrc_getcaps (GstBaseSrc * bsrc)
+{
+  GstElementClass *element_class;
+  GstPadTemplate *pad_template;
+  GstAlsaSrc *src;
+  GstCaps *caps;
+
+  src = GST_ALSA_SRC (bsrc);
+
+  if (src->handle == NULL) {
+    GST_DEBUG_OBJECT (src, "device not open, using template caps");
+    return NULL;                /* base class will get template caps for us */
+  }
+
+  if (src->cached_caps) {
+    GST_LOG_OBJECT (src, "Returning cached caps");
+    return gst_caps_ref (src->cached_caps);
+  }
+
+  element_class = GST_ELEMENT_GET_CLASS (src);
+  pad_template = gst_element_class_get_pad_template (element_class, "src");
+  g_return_val_if_fail (pad_template != NULL, NULL);
+
+  caps = gst_alsa_probe_supported_formats (GST_OBJECT (src), src->handle,
+      gst_pad_template_get_caps (pad_template));
+
+  if (caps) {
+    src->cached_caps = gst_caps_ref (caps);
+  }
+
+  GST_INFO_OBJECT (src, "returning caps %" GST_PTR_FORMAT, caps);
+
+  return caps;
+}
+
 static int
 set_hwparams (GstAlsaSrc * alsa)
 {
@@ -257,7 +348,7 @@ set_hwparams (GstAlsaSrc * alsa)
   gint err, dir;
   snd_pcm_hw_params_t *params;
 
-  snd_pcm_hw_params_alloca (&params);
+  snd_pcm_hw_params_malloc (&params);
 
   /* choose all parameters */
   CHECK (snd_pcm_hw_params_any (alsa->handle, params), no_config);
@@ -272,7 +363,7 @@ set_hwparams (GstAlsaSrc * alsa)
       no_channels);
   /* set the stream rate */
   rrate = alsa->rate;
-  CHECK (snd_pcm_hw_params_set_rate_near (alsa->handle, params, &rrate, 0),
+  CHECK (snd_pcm_hw_params_set_rate_near (alsa->handle, params, &rrate, NULL),
       no_rate);
   if (rrate != alsa->rate)
     goto rate_match;
@@ -297,6 +388,7 @@ set_hwparams (GstAlsaSrc * alsa)
   CHECK (snd_pcm_hw_params_get_period_size (params, &alsa->period_size, &dir),
       period_size);
 
+  snd_pcm_hw_params_free (params);
   return 0;
 
   /* ERRORS */
@@ -305,18 +397,21 @@ no_config:
     GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS, (NULL),
         ("Broken configuration for recording: no configurations available: %s",
             snd_strerror (err)));
+    snd_pcm_hw_params_free (params);
     return err;
   }
 wrong_access:
   {
     GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS, (NULL),
         ("Access type not available for recording: %s", snd_strerror (err)));
+    snd_pcm_hw_params_free (params);
     return err;
   }
 no_sample_format:
   {
     GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS, (NULL),
         ("Sample format not available for recording: %s", snd_strerror (err)));
+    snd_pcm_hw_params_free (params);
     return err;
   }
 no_channels:
@@ -334,6 +429,7 @@ no_channels:
           alsa->channels);
     GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS, (msg), (snd_strerror (err)));
     g_free (msg);
+    snd_pcm_hw_params_free (params);
     return err;
   }
 no_rate:
@@ -341,12 +437,14 @@ no_rate:
     GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS, (NULL),
         ("Rate %iHz not available for recording: %s",
             alsa->rate, snd_strerror (err)));
+    snd_pcm_hw_params_free (params);
     return err;
   }
 rate_match:
   {
     GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS, (NULL),
         ("Rate doesn't match (requested %iHz, get %iHz)", alsa->rate, err));
+    snd_pcm_hw_params_free (params);
     return -EINVAL;
   }
 buffer_time:
@@ -354,12 +452,14 @@ buffer_time:
     GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS, (NULL),
         ("Unable to set buffer time %i for recording: %s",
             alsa->buffer_time, snd_strerror (err)));
+    snd_pcm_hw_params_free (params);
     return err;
   }
 buffer_size:
   {
     GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS, (NULL),
         ("Unable to get buffer size for recording: %s", snd_strerror (err)));
+    snd_pcm_hw_params_free (params);
     return err;
   }
 period_time:
@@ -367,18 +467,21 @@ period_time:
     GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS, (NULL),
         ("Unable to set period time %i for recording: %s", alsa->period_time,
             snd_strerror (err)));
+    snd_pcm_hw_params_free (params);
     return err;
   }
 period_size:
   {
     GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS, (NULL),
         ("Unable to get period size for recording: %s", snd_strerror (err)));
+    snd_pcm_hw_params_free (params);
     return err;
   }
 set_hw_params:
   {
     GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS, (NULL),
         ("Unable to set hw params for recording: %s", snd_strerror (err)));
+    snd_pcm_hw_params_free (params);
     return err;
   }
 }
@@ -389,27 +492,28 @@ set_swparams (GstAlsaSrc * alsa)
   int err;
   snd_pcm_sw_params_t *params;
 
-  snd_pcm_sw_params_alloca (&params);
+  snd_pcm_sw_params_malloc (&params);
 
   /* get the current swparams */
   CHECK (snd_pcm_sw_params_current (alsa->handle, params), no_config);
-  /* start the transfer when the buffer is almost full: */
-  /* (buffer_size / avail_min) * avail_min */
-#if 0
-  CHECK (snd_pcm_sw_params_set_start_threshold (alsa->handle, params,
-          (alsa->buffer_size / alsa->period_size) * alsa->period_size),
-      start_threshold);
-
   /* allow the transfer when at least period_size samples can be processed */
   CHECK (snd_pcm_sw_params_set_avail_min (alsa->handle, params,
           alsa->period_size), set_avail);
-#endif
+  /* start the transfer on first read */
+  CHECK (snd_pcm_sw_params_set_start_threshold (alsa->handle, params,
+          0), start_threshold);
+
+#if GST_CHECK_ALSA_VERSION(1,0,16)
+  /* snd_pcm_sw_params_set_xfer_align() is deprecated, alignment is always 1 */
+#else
   /* align all transfers to 1 sample */
   CHECK (snd_pcm_sw_params_set_xfer_align (alsa->handle, params, 1), set_align);
+#endif
 
   /* write the parameters to the recording device */
   CHECK (snd_pcm_sw_params (alsa->handle, params), set_sw_params);
 
+  snd_pcm_sw_params_free (params);
   return 0;
 
   /* ERRORS */
@@ -418,33 +522,38 @@ no_config:
     GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS, (NULL),
         ("Unable to determine current swparams for playback: %s",
             snd_strerror (err)));
+    snd_pcm_sw_params_free (params);
     return err;
   }
-#if 0
 start_threshold:
   {
     GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS, (NULL),
         ("Unable to set start threshold mode for playback: %s",
             snd_strerror (err)));
+    snd_pcm_sw_params_free (params);
     return err;
   }
 set_avail:
   {
     GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS, (NULL),
         ("Unable to set avail min for playback: %s", snd_strerror (err)));
+    snd_pcm_sw_params_free (params);
     return err;
   }
-#endif
+#if !GST_CHECK_ALSA_VERSION(1,0,16)
 set_align:
   {
     GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS, (NULL),
         ("Unable to set transfer align for playback: %s", snd_strerror (err)));
+    snd_pcm_sw_params_free (params);
     return err;
   }
+#endif
 set_sw_params:
   {
     GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS, (NULL),
         ("Unable to set sw params for playback: %s", snd_strerror (err)));
+    snd_pcm_sw_params_free (params);
     return err;
   }
 }
@@ -520,10 +629,15 @@ gst_alsasrc_open (GstAudioSrc * asrc)
 open_error:
   {
     if (err == -EBUSY) {
-      GST_ELEMENT_ERROR (alsa, RESOURCE, BUSY, (NULL), (NULL));
+      GST_ELEMENT_ERROR (alsa, RESOURCE, BUSY,
+          (_("Could not open audio device for recording. "
+                  "Device is being used by another application.")),
+          ("Device '%s' is busy", alsa->device));
     } else {
       GST_ELEMENT_ERROR (alsa, RESOURCE, OPEN_READ,
-          (NULL), ("Recording open error: %s", snd_strerror (err)));
+          (_("Could not open audio device for recording.")),
+          ("Recording open error on device '%s': %s", alsa->device,
+              snd_strerror (err)));
     }
     return FALSE;
   }
@@ -638,6 +752,8 @@ gst_alsasrc_close (GstAudioSrc * asrc)
     alsa->mixer = NULL;
   }
 
+  gst_caps_replace (&alsa->cached_caps, NULL);
+
   return TRUE;
 }
 
@@ -645,14 +761,15 @@ gst_alsasrc_close (GstAudioSrc * asrc)
  *   Underrun and suspend recovery
  */
 static gint
-xrun_recovery (snd_pcm_t * handle, gint err)
+xrun_recovery (GstAlsaSrc * alsa, snd_pcm_t * handle, gint err)
 {
-  GST_DEBUG ("xrun recovery %d", err);
+  GST_DEBUG_OBJECT (alsa, "xrun recovery %d", err);
 
   if (err == -EPIPE) {          /* under-run */
     err = snd_pcm_prepare (handle);
     if (err < 0)
-      GST_WARNING ("Can't recovery from underrun, prepare failed: %s",
+      GST_WARNING_OBJECT (alsa,
+          "Can't recovery from underrun, prepare failed: %s",
           snd_strerror (err));
     return 0;
   } else if (err == -ESTRPIPE) {
@@ -662,7 +779,8 @@ xrun_recovery (snd_pcm_t * handle, gint err)
     if (err < 0) {
       err = snd_pcm_prepare (handle);
       if (err < 0)
-        GST_WARNING ("Can't recovery from suspend, prepare failed: %s",
+        GST_WARNING_OBJECT (alsa,
+            "Can't recovery from suspend, prepare failed: %s",
             snd_strerror (err));
     }
     return 0;
@@ -683,12 +801,13 @@ gst_alsasrc_read (GstAudioSrc * asrc, gpointer data, guint length)
   cptr = length / alsa->bytes_per_sample;
   ptr = data;
 
+  GST_ALSA_SRC_LOCK (asrc);
   while (cptr > 0) {
     if ((err = snd_pcm_readi (alsa->handle, ptr, cptr)) < 0) {
       if (err == -EAGAIN) {
         GST_DEBUG_OBJECT (asrc, "Read error: %s", snd_strerror (err));
         continue;
-      } else if (xrun_recovery (alsa->handle, err) < 0) {
+      } else if (xrun_recovery (alsa, alsa->handle, err) < 0) {
         goto read_error;
       }
       continue;
@@ -697,10 +816,13 @@ gst_alsasrc_read (GstAudioSrc * asrc, gpointer data, guint length)
     ptr += err * alsa->channels;
     cptr -= err;
   }
+  GST_ALSA_SRC_UNLOCK (asrc);
+
   return length - cptr;
 
 read_error:
   {
+    GST_ALSA_SRC_UNLOCK (asrc);
     return length;              /* skip one period */
   }
 }
@@ -710,10 +832,15 @@ gst_alsasrc_delay (GstAudioSrc * asrc)
 {
   GstAlsaSrc *alsa;
   snd_pcm_sframes_t delay;
+  int res;
 
   alsa = GST_ALSA_SRC (asrc);
 
-  snd_pcm_delay (alsa->handle, &delay);
+  res = snd_pcm_delay (alsa->handle, &delay);
+  if (G_UNLIKELY (res < 0)) {
+    GST_DEBUG_OBJECT (alsa, "snd_pcm_delay returned %d", res);
+    delay = 0;
+  }
 
   return CLAMP (delay, 0, alsa->buffer_size);
 }
@@ -721,30 +848,34 @@ gst_alsasrc_delay (GstAudioSrc * asrc)
 static void
 gst_alsasrc_reset (GstAudioSrc * asrc)
 {
-
-#if 0
   GstAlsaSrc *alsa;
   gint err;
 
   alsa = GST_ALSA_SRC (asrc);
 
+  GST_ALSA_SRC_LOCK (asrc);
+  GST_DEBUG_OBJECT (alsa, "drop");
   CHECK (snd_pcm_drop (alsa->handle), drop_error);
+  GST_DEBUG_OBJECT (alsa, "prepare");
   CHECK (snd_pcm_prepare (alsa->handle), prepare_error);
+  GST_DEBUG_OBJECT (alsa, "reset done");
+  GST_ALSA_SRC_UNLOCK (asrc);
 
   return;
 
   /* ERRORS */
 drop_error:
   {
-    GST_ELEMENT_ERROR (alsa, RESOURCE, OPEN_READ,
-        ("alsa-reset: pcm drop error: %s", snd_strerror (err)), (NULL));
+    GST_ERROR_OBJECT (alsa, "alsa-reset: pcm drop error: %s",
+        snd_strerror (err));
+    GST_ALSA_SRC_UNLOCK (asrc);
     return;
   }
 prepare_error:
   {
-    GST_ELEMENT_ERROR (alsa, RESOURCE, OPEN_READ,
-        ("alsa-reset: pcm prepare error: %s", snd_strerror (err)), (NULL));
+    GST_ERROR_OBJECT (alsa, "alsa-reset: pcm prepare error: %s",
+        snd_strerror (err));
+    GST_ALSA_SRC_UNLOCK (asrc);
     return;
   }
-#endif
 }
