@@ -45,6 +45,8 @@ struct _GstBaseRTPPayloadPrivate
   gboolean seqnum_offset_random;
   gboolean ssrc_random;
   guint16 next_seqnum;
+
+  GstClockTime rt_base;
 };
 
 /* BaseRTPPayload signals and args */
@@ -60,7 +62,7 @@ enum
  * 1432 bytes or so.  And that should be adjusted downward further for other
  * encapsulations like PPPoE, so 1400 at most.
  */
-#define DEFAULT_MTU                     1024
+#define DEFAULT_MTU                     1400
 #define DEFAULT_PT                      96
 #define DEFAULT_SSRC                    -1
 #define DEFAULT_TIMESTAMP_OFFSET        -1
@@ -89,6 +91,7 @@ static void gst_basertppayload_init (GstBaseRTPPayload * basertppayload,
 static void gst_basertppayload_finalize (GObject * object);
 
 static gboolean gst_basertppayload_setcaps (GstPad * pad, GstCaps * caps);
+static GstCaps *gst_basertppayload_getcaps (GstPad * pad);
 static gboolean gst_basertppayload_event (GstPad * pad, GstEvent * event);
 static GstFlowReturn gst_basertppayload_chain (GstPad * pad,
     GstBuffer * buffer);
@@ -228,6 +231,8 @@ gst_basertppayload_init (GstBaseRTPPayload * basertppayload, gpointer g_class)
   basertppayload->sinkpad = gst_pad_new_from_template (templ, "sink");
   gst_pad_set_setcaps_function (basertppayload->sinkpad,
       gst_basertppayload_setcaps);
+  gst_pad_set_getcaps_function (basertppayload->sinkpad,
+      gst_basertppayload_getcaps);
   gst_pad_set_event_function (basertppayload->sinkpad,
       gst_basertppayload_event);
   gst_pad_set_chain_function (basertppayload->sinkpad,
@@ -297,6 +302,35 @@ gst_basertppayload_setcaps (GstPad * pad, GstCaps * caps)
   return ret;
 }
 
+static GstCaps *
+gst_basertppayload_getcaps (GstPad * pad)
+{
+  GstBaseRTPPayload *basertppayload;
+  GstBaseRTPPayloadClass *basertppayload_class;
+  GstCaps *caps = NULL;
+
+  GST_DEBUG_OBJECT (pad, "getting caps");
+
+  basertppayload = GST_BASE_RTP_PAYLOAD (gst_pad_get_parent (pad));
+  basertppayload_class = GST_BASE_RTP_PAYLOAD_GET_CLASS (basertppayload);
+
+  if (basertppayload_class->get_caps)
+    caps = basertppayload_class->get_caps (basertppayload, pad);
+
+  if (!caps) {
+    caps = GST_PAD_TEMPLATE_CAPS (GST_PAD_PAD_TEMPLATE (pad));
+    GST_DEBUG_OBJECT (pad,
+        "using pad template %p with caps %p %" GST_PTR_FORMAT,
+        GST_PAD_PAD_TEMPLATE (pad), caps, caps);
+
+    caps = gst_caps_ref (caps);
+  }
+
+  gst_object_unref (basertppayload);
+
+  return caps;
+}
+
 static gboolean
 gst_basertppayload_event (GstPad * pad, GstEvent * event)
 {
@@ -332,8 +366,9 @@ gst_basertppayload_event (GstPad * pad, GstEvent * event)
           &position);
       gst_segment_set_newsegment (&basertppayload->segment, update, rate, fmt,
           start, stop, position);
-    }
+
       /* fallthrough */
+    }
     default:
       res = gst_pad_event_default (pad, event);
       break;
@@ -371,10 +406,23 @@ no_function:
     GST_ELEMENT_ERROR (basertppayload, STREAM, NOT_IMPLEMENTED, (NULL),
         ("subclass did not implement handle_buffer function"));
     gst_object_unref (basertppayload);
+    gst_buffer_unref (buffer);
     return GST_FLOW_ERROR;
   }
 }
 
+/**
+ * gst_basertppayload_set_options:
+ * @payload: a #GstBaseRTPPayload
+ * @media: the media type (typically "audio" or "video")
+ * @dynamic: if the payload type is dynamic
+ * @encoding_name: the encoding name 
+ * @clock_rate: the clock rate of the media
+ *
+ * Set the rtp options of the payloader. These options will be set in the caps
+ * of the payloader. Subclasses must call this method before calling
+ * gst_basertppayload_push() or gst_basertppayload_set_outcaps().
+ */
 void
 gst_basertppayload_set_options (GstBaseRTPPayload * payload,
     gchar * media, gboolean dynamic, gchar * encoding_name, guint32 clock_rate)
@@ -390,29 +438,145 @@ gst_basertppayload_set_options (GstBaseRTPPayload * payload,
   payload->clock_rate = clock_rate;
 }
 
+static gboolean
+copy_fixed (GQuark field_id, const GValue * value, GstStructure * dest)
+{
+  if (gst_value_is_fixed (value)) {
+    gst_structure_id_set_value (dest, field_id, value);
+  }
+  return TRUE;
+}
+
+/**
+ * gst_basertppayload_set_outcaps:
+ * @payload: a #GstBaseRTPPayload
+ * @fieldname: the first field name or %NULL
+ * @...: field values
+ *
+ * Configure the output caps with the optional parameters.
+ *
+ * Variable arguments should be in the form field name, field type
+ * (as a GType), value(s).  The last variable argument should be NULL.
+ *
+ * Returns: %TRUE if the caps could be set.
+ */
 gboolean
 gst_basertppayload_set_outcaps (GstBaseRTPPayload * payload, gchar * fieldname,
     ...)
 {
-  GstCaps *srccaps;
-  GstStructure *s;
+  GstCaps *srccaps, *peercaps;
 
+  /* fill in the defaults, there properties cannot be negotiated. */
   srccaps = gst_caps_new_simple ("application/x-rtp",
       "media", G_TYPE_STRING, payload->media,
-      "payload", G_TYPE_INT, GST_BASE_RTP_PAYLOAD_PT (payload),
       "clock-rate", G_TYPE_INT, payload->clock_rate,
-      "encoding-name", G_TYPE_STRING, payload->encoding_name,
-      "ssrc", G_TYPE_UINT, payload->current_ssrc,
-      "clock-base", G_TYPE_UINT, payload->ts_base,
-      "seqnum-base", G_TYPE_UINT, payload->seqnum_base, NULL);
-  s = gst_caps_get_structure (srccaps, 0);
+      "encoding-name", G_TYPE_STRING, payload->encoding_name, NULL);
+
+  GST_DEBUG_OBJECT (payload, "defaults: %" GST_PTR_FORMAT, srccaps);
 
   if (fieldname) {
     va_list varargs;
 
+    /* override with custom properties */
     va_start (varargs, fieldname);
-    gst_structure_set_valist (s, fieldname, varargs);
+    gst_caps_set_simple_valist (srccaps, fieldname, varargs);
     va_end (varargs);
+
+    GST_DEBUG_OBJECT (payload, "custom added: %" GST_PTR_FORMAT, srccaps);
+  }
+
+  /* the peer caps can override some of the defaults */
+  peercaps = gst_pad_peer_get_caps (payload->srcpad);
+  if (peercaps == NULL) {
+    /* no peer caps, just add the other properties */
+    gst_caps_set_simple (srccaps,
+        "payload", G_TYPE_INT, GST_BASE_RTP_PAYLOAD_PT (payload),
+        "ssrc", G_TYPE_UINT, payload->current_ssrc,
+        "clock-base", G_TYPE_UINT, payload->ts_base,
+        "seqnum-base", G_TYPE_UINT, payload->seqnum_base, NULL);
+
+    GST_DEBUG_OBJECT (payload, "no peer caps: %" GST_PTR_FORMAT, srccaps);
+  } else {
+    GstCaps *temp;
+    GstStructure *s, *d;
+    const GValue *value;
+    gint pt;
+
+    /* peer provides caps we can use to fixate, intersect. This always returns a
+     * writable caps. */
+    temp = gst_caps_intersect (srccaps, peercaps);
+    gst_caps_unref (srccaps);
+    gst_caps_unref (peercaps);
+
+    /* now fixate, start by taking the first caps */
+    gst_caps_truncate (temp);
+
+    /* get first structure */
+    s = gst_caps_get_structure (temp, 0);
+
+    if (gst_structure_get_int (s, "payload", &pt)) {
+      /* use peer pt */
+      GST_BASE_RTP_PAYLOAD_PT (payload) = pt;
+      GST_LOG_OBJECT (payload, "using peer pt %d", pt);
+    } else {
+      if (gst_structure_has_field (s, "payload")) {
+        /* can only fixate if there is a field */
+        gst_structure_fixate_field_nearest_int (s, "payload",
+            GST_BASE_RTP_PAYLOAD_PT (payload));
+        gst_structure_get_int (s, "payload", &pt);
+        GST_LOG_OBJECT (payload, "using peer pt %d", pt);
+      } else {
+        /* no pt field, use the internal pt */
+        pt = GST_BASE_RTP_PAYLOAD_PT (payload);
+        gst_structure_set (s, "payload", G_TYPE_INT, pt, NULL);
+        GST_LOG_OBJECT (payload, "using internal pt %d", pt);
+      }
+    }
+
+    if (gst_structure_has_field_typed (s, "ssrc", G_TYPE_UINT)) {
+      value = gst_structure_get_value (s, "ssrc");
+      payload->current_ssrc = g_value_get_uint (value);
+      GST_LOG_OBJECT (payload, "using peer ssrc %08x", payload->current_ssrc);
+    } else {
+      /* FIXME, fixate_nearest_uint would be even better */
+      gst_structure_set (s, "ssrc", G_TYPE_UINT, payload->current_ssrc, NULL);
+      GST_LOG_OBJECT (payload, "using internal ssrc %08x",
+          payload->current_ssrc);
+    }
+
+    if (gst_structure_has_field_typed (s, "clock-base", G_TYPE_UINT)) {
+      value = gst_structure_get_value (s, "clock-base");
+      payload->ts_base = g_value_get_uint (value);
+      GST_LOG_OBJECT (payload, "using peer clock-base %u", payload->ts_base);
+    } else {
+      /* FIXME, fixate_nearest_uint would be even better */
+      gst_structure_set (s, "clock-base", G_TYPE_UINT, payload->ts_base, NULL);
+      GST_LOG_OBJECT (payload, "using internal clock-base %u",
+          payload->ts_base);
+    }
+    if (gst_structure_has_field_typed (s, "seqnum-base", G_TYPE_UINT)) {
+      value = gst_structure_get_value (s, "seqnum-base");
+      payload->seqnum_base = g_value_get_uint (value);
+      GST_LOG_OBJECT (payload, "using peer seqnum-base %u",
+          payload->seqnum_base);
+    } else {
+      /* FIXME, fixate_nearest_uint would be even better */
+      gst_structure_set (s, "seqnum-base", G_TYPE_UINT, payload->seqnum_base,
+          NULL);
+      GST_LOG_OBJECT (payload, "using internal seqnum-base %u",
+          payload->seqnum_base);
+    }
+
+    /* make the target caps by copying over all the fixed caps, removing the
+     * unfixed caps. */
+    srccaps = gst_caps_new_simple (gst_structure_get_name (s), NULL);
+    d = gst_caps_get_structure (srccaps, 0);
+
+    gst_structure_foreach (s, (GstStructureForeachFunc) copy_fixed, d);
+
+    gst_caps_unref (temp);
+
+    GST_DEBUG_OBJECT (payload, "with peer caps: %" GST_PTR_FORMAT, srccaps);
   }
 
   gst_pad_set_caps (GST_BASE_RTP_PAYLOAD_SRCPAD (payload), srccaps);
@@ -421,6 +585,18 @@ gst_basertppayload_set_outcaps (GstBaseRTPPayload * payload, gchar * fieldname,
   return TRUE;
 }
 
+/**
+ * gst_basertppayload_is_filled:
+ * @payload: a #GstBaseRTPPayload
+ * @size: the size of the packet
+ * @duration: the duration of the packet
+ *
+ * Check if the packet with @size and @duration would exceed the configure
+ * maximum size.
+ *
+ * Returns: %TRUE if the packet of @size and @duration would exceed the
+ * configured MTU or max_ptime.
+ */
 gboolean
 gst_basertppayload_is_filled (GstBaseRTPPayload * payload,
     guint size, GstClockTime duration)
@@ -434,12 +610,24 @@ gst_basertppayload_is_filled (GstBaseRTPPayload * payload,
   return FALSE;
 }
 
+/**
+ * gst_basertppayload_push:
+ * @payload: a #GstBaseRTPPayload
+ * @buffer: a #GstBuffer
+ *
+ * Push @buffer to the peer element of the payloader. The SSRC, payload type,
+ * seqnum and timestamp of the RTP buffer will be updated first.
+ * 
+ * This function takes ownership of @buffer.
+ *
+ * Returns: a #GstFlowReturn.
+ */
 GstFlowReturn
 gst_basertppayload_push (GstBaseRTPPayload * payload, GstBuffer * buffer)
 {
   GstFlowReturn res;
   GstClockTime timestamp;
-  guint32 ts;
+  guint32 rtptime;
   GstBaseRTPPayloadPrivate *priv;
 
   if (payload->clock_rate == 0)
@@ -454,36 +642,49 @@ gst_basertppayload_push (GstBaseRTPPayload * payload, GstBuffer * buffer)
   /* update first, so that the property is set to the last
    * seqnum pushed */
   payload->seqnum = priv->next_seqnum;
-  GST_LOG_OBJECT (payload, "setting RTP seqnum %d", payload->seqnum);
   gst_rtp_buffer_set_seq (buffer, payload->seqnum);
 
   /* can wrap around, which is perfectly fine */
   priv->next_seqnum++;
 
   /* add our random offset to the timestamp */
-  ts = payload->ts_base;
+  rtptime = payload->ts_base;
 
   timestamp = GST_BUFFER_TIMESTAMP (buffer);
   if (GST_CLOCK_TIME_IS_VALID (timestamp)) {
     gint64 rtime;
 
-    rtime =
-        gst_segment_to_running_time (&payload->segment, GST_FORMAT_TIME,
+    rtime = gst_segment_to_running_time (&payload->segment, GST_FORMAT_TIME,
         timestamp);
+
+    /* take first timestamp as base, we want to calculate the RTP timestamp
+     * starting from the ts_base */
+    if (priv->rt_base == -1) {
+      priv->rt_base = rtime;
+      GST_LOG_OBJECT (payload, "first timestamp %" GST_TIME_FORMAT,
+          GST_TIME_ARGS (rtime));
+    }
+    rtime -= priv->rt_base;
+
     rtime = gst_util_uint64_scale_int (rtime, payload->clock_rate, GST_SECOND);
 
-    ts += rtime;
+    /* add running_time in clock-rate units to the base timestamp */
+    rtptime += rtime;
+  } else {
+    /* no timestamp to convert, take previous timestamp */
+    rtptime = payload->timestamp;
   }
-  GST_LOG_OBJECT (payload, "setting RTP timestamp %u", (guint) ts);
-  gst_rtp_buffer_set_timestamp (buffer, ts);
+  gst_rtp_buffer_set_timestamp (buffer, rtptime);
 
-  payload->timestamp = ts;
+  payload->timestamp = rtptime;
 
   /* set caps */
   gst_buffer_set_caps (buffer, GST_PAD_CAPS (payload->srcpad));
 
-  GST_DEBUG_OBJECT (payload, "Pushing packet size %d, seq=%d, ts=%u",
-      GST_BUFFER_SIZE (buffer), payload->seqnum - 1, ts);
+  GST_LOG_OBJECT (payload,
+      "Pushing packet size %d, seq=%d, rtptime=%u, timestamp %" GST_TIME_FORMAT,
+      GST_BUFFER_SIZE (buffer), payload->seqnum, rtptime,
+      GST_TIME_ARGS (timestamp));
 
   res = gst_pad_push (payload->srcpad, buffer);
 
@@ -493,7 +694,7 @@ gst_basertppayload_push (GstBaseRTPPayload * payload, GstBuffer * buffer)
 no_rate:
   {
     GST_ELEMENT_ERROR (payload, STREAM, NOT_IMPLEMENTED, (NULL),
-        ("subclass did not specify clock_rate"));
+        ("subclass did not specify clock-rate"));
     gst_buffer_unref (buffer);
     return GST_FLOW_ERROR;
   }
@@ -632,6 +833,8 @@ gst_basertppayload_change_state (GstElement * element,
         basertppayload->ts_base = g_rand_int (basertppayload->ts_rand);
       else
         basertppayload->ts_base = basertppayload->ts_offset;
+
+      priv->rt_base = -1;
       break;
     default:
       break;
