@@ -25,6 +25,29 @@
 
 G_BEGIN_DECLS
 
+/**
+ * GstVideoFormat:
+ * @GST_VIDEO_FORMAT_UNKNOWN,
+ * @GST_VIDEO_FORMAT_I420: planar 4:2:0 YUV
+ * @GST_VIDEO_FORMAT_YV12: planar 4:2:0 YVU (like I420 but UV planes swapped)
+ * @GST_VIDEO_FORMAT_YUY2: packed 4:2:2 YUV (Y0-U0-Y1-V0 Y2-U2-Y3-V2 Y4 ...)
+ * @GST_VIDEO_FORMAT_UYVY: packed 4:2:2 YUV (U0-Y0-V0-Y1 U2-Y2-V2-Y3 U4 ...) 
+ * @GST_VIDEO_FORMAT_AYUV: packed 4:4:4 YUV with alpha channel (A0-Y0-U0-V0 ...)
+ * @GST_VIDEO_FORMAT_RGBx:
+ * @GST_VIDEO_FORMAT_BGRx:
+ * @GST_VIDEO_FORMAT_xRGB:
+ * @GST_VIDEO_FORMAT_xBGR:
+ * @GST_VIDEO_FORMAT_RGBA:
+ * @GST_VIDEO_FORMAT_BGRA:
+ * @GST_VIDEO_FORMAT_ARGB:
+ * @GST_VIDEO_FORMAT_ABGR:
+ * @GST_VIDEO_FORMAT_RGB:
+ * @GST_VIDEO_FORMAT_BGR:
+ * @GST_VIDEO_FORMAT_Y41B: planar 4:1:1 YUV (Since: 0.10.18)
+ * @GST_VIDEO_FORMAT_Y42B: planar 4:2:2 YUV (Since: 0.10.18)
+ *
+ * Enum value describing the most common video formats.
+ */
 typedef enum {
   GST_VIDEO_FORMAT_UNKNOWN,
   GST_VIDEO_FORMAT_I420,
@@ -41,7 +64,9 @@ typedef enum {
   GST_VIDEO_FORMAT_ARGB,
   GST_VIDEO_FORMAT_ABGR,
   GST_VIDEO_FORMAT_RGB,
-  GST_VIDEO_FORMAT_BGR
+  GST_VIDEO_FORMAT_BGR,
+  GST_VIDEO_FORMAT_Y41B,
+  GST_VIDEO_FORMAT_Y42B
 } GstVideoFormat;
 
 #define GST_VIDEO_BYTE1_MASK_32  "0xFF000000"
