@@ -44,17 +44,17 @@ static GstStaticPadTemplate src_templ = GST_STATIC_PAD_TEMPLATE ("src",
     GST_STATIC_CAPS ("text/x-pango-markup")
     );
 
-GST_BOILERPLATE (GstSsaParse, gst_ssa_parse, GstElement, GST_TYPE_ELEMENT)
+GST_BOILERPLATE (GstSsaParse, gst_ssa_parse, GstElement, GST_TYPE_ELEMENT);
 
-     static GstStateChangeReturn gst_ssa_parse_change_state (GstElement *
+static GstStateChangeReturn gst_ssa_parse_change_state (GstElement *
     element, GstStateChange transition);
-     static gboolean gst_ssa_parse_setcaps (GstPad * sinkpad, GstCaps * caps);
-     static gboolean gst_ssa_parse_src_event (GstPad * pad, GstEvent * event);
-     static gboolean gst_ssa_parse_sink_event (GstPad * pad, GstEvent * event);
-     static GstFlowReturn gst_ssa_parse_chain (GstPad * sinkpad,
-    GstBuffer * buf);
+static gboolean gst_ssa_parse_setcaps (GstPad * sinkpad, GstCaps * caps);
+static gboolean gst_ssa_parse_src_event (GstPad * pad, GstEvent * event);
+static gboolean gst_ssa_parse_sink_event (GstPad * pad, GstEvent * event);
+static GstFlowReturn gst_ssa_parse_chain (GstPad * sinkpad, GstBuffer * buf);
 
-     static void gst_ssa_parse_base_init (gpointer klass)
+static void
+gst_ssa_parse_base_init (gpointer klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
   static GstElementDetails ssa_parse_details = {
@@ -138,7 +138,7 @@ gst_ssa_parse_setcaps (GstPad * sinkpad, GstCaps * caps)
   GstSsaParse *parse = GST_SSA_PARSE (GST_PAD_PARENT (sinkpad));
   const GValue *val;
   GstStructure *s;
-  const gchar bom_utf8[] = { 0xEF, 0xBB, 0xBF };
+  const guchar bom_utf8[] = { 0xEF, 0xBB, 0xBF };
   GstBuffer *priv;
   gchar *data;
   guint size;
@@ -204,11 +204,38 @@ gst_ssa_parse_remove_override_codes (GstSsaParse * parse, gchar * txt)
     removed_any = TRUE;
   }
 
+  /* these may occur outside of curly brackets. We don't handle the different
+   * wrapping modes yet, so just remove these markers from the text for now */
+  while ((t = strstr (txt, "\\n"))) {
+    t[0] = ' ';
+    t[1] = '\n';
+  }
+  while ((t = strstr (txt, "\\N"))) {
+    t[0] = ' ';
+    t[1] = '\n';
+  }
+  while ((t = strstr (txt, "\\h"))) {
+    t[0] = ' ';
+    t[1] = ' ';
+  }
+
   return removed_any;
 }
 
+/**
+ * gst_ssa_parse_push_line:
+ * @parse: caller element
+ * @txt: text to push
+ * @start: timestamp for the buffer
+ * @duration: duration for the buffer
+ *
+ * Parse the text in a buffer with the given properties and
+ * push it to the srcpad of the @parse element
+ *
+ * Returns: result of the push of the created buffer
+ */
 static GstFlowReturn
-gst_ssa_parse_parse_line (GstSsaParse * parse, gchar * txt,
+gst_ssa_parse_push_line (GstSsaParse * parse, gchar * txt,
     GstClockTime start, GstClockTime duration)
 {
   GstFlowReturn ret;
@@ -240,8 +267,11 @@ gst_ssa_parse_parse_line (GstSsaParse * parse, gchar * txt,
   escaped = g_markup_printf_escaped ("%s", t);
 
   len = strlen (escaped);
+
+  /* allocate enough for a terminating NUL, but don't include it in buf size */
   buf = gst_buffer_new_and_alloc (len + 1);
-  memcpy (GST_BUFFER_DATA (buf), escaped, len + 1);     /* incl. terminating NUL */
+  memcpy (GST_BUFFER_DATA (buf), escaped, len + 1);
+  GST_BUFFER_SIZE (buf) = len;
   g_free (escaped);
 
   GST_BUFFER_TIMESTAMP (buf) = start;
@@ -256,7 +286,8 @@ gst_ssa_parse_parse_line (GstSsaParse * parse, gchar * txt,
   ret = gst_pad_push (parse->srcpad, buf);
 
   if (ret != GST_FLOW_OK) {
-    GST_DEBUG_OBJECT (parse, "Push returned flow %s", gst_flow_get_name (ret));
+    GST_DEBUG_OBJECT (parse, "Push of text '%s' returned flow %s", txt,
+        gst_flow_get_name (ret));
   }
 
   return ret;
@@ -270,6 +301,9 @@ gst_ssa_parse_chain (GstPad * sinkpad, GstBuffer * buf)
   GstClockTime ts;
   gchar *txt;
 
+  if (G_UNLIKELY (!parse->framed))
+    goto not_framed;
+
   /* make double-sure it's 0-terminated and all */
   txt = g_strndup ((gchar *) GST_BUFFER_DATA (buf), GST_BUFFER_SIZE (buf));
 
@@ -277,21 +311,26 @@ gst_ssa_parse_chain (GstPad * sinkpad, GstBuffer * buf)
     return GST_FLOW_UNEXPECTED;
 
   ts = GST_BUFFER_TIMESTAMP (buf);
-  ret = gst_ssa_parse_parse_line (parse, txt, ts, GST_BUFFER_DURATION (buf));
+  ret = gst_ssa_parse_push_line (parse, txt, ts, GST_BUFFER_DURATION (buf));
 
-  if (ret != GST_FLOW_OK) {
-    GST_WARNING_OBJECT (parse, "Failed to parse dialog line '%s'", txt);
-    if (GST_CLOCK_TIME_IS_VALID (ts)) {
-      /* just advance time without sending anything */
-      gst_pad_push_event (parse->srcpad,
-          gst_event_new_new_segment (TRUE, 1.0, GST_FORMAT_TIME, ts, -1, ts));
-      ret = GST_FLOW_OK;
-    }
+  if (ret != GST_FLOW_OK && GST_CLOCK_TIME_IS_VALID (ts)) {
+    /* just advance time without sending anything */
+    gst_pad_push_event (parse->srcpad,
+        gst_event_new_new_segment (TRUE, 1.0, GST_FORMAT_TIME, ts, -1, ts));
+    ret = GST_FLOW_OK;
   }
 
   g_free (txt);
 
   return ret;
+
+/* ERRORS */
+not_framed:
+  {
+    GST_ELEMENT_ERROR (parse, STREAM, FORMAT, (NULL),
+        ("Only SSA subtitles embedded in containers are supported"));
+    return GST_FLOW_NOT_NEGOTIATED;
+  }
 }
 
 static GstStateChangeReturn
