@@ -32,6 +32,7 @@
 #include "gstssaparse.h"
 #include "samiparse.h"
 #include "tmplayerparse.h"
+#include "mpl2parse.h"
 
 GST_DEBUG_CATEGORY (sub_parse_debug);
 
@@ -58,12 +59,12 @@ GST_ELEMENT_DETAILS ("Subtitle parser",
     "Gustavo J. A. M. Carneiro <gjc@inescporto.pt>\n"
     "Ronald S. Bultje <rbultje@ronald.bitfreak.net>");
 
-#ifndef GST_DISABLE_LOADSAVE_REGISTRY
+#ifndef GST_DISABLE_XML
 static GstStaticPadTemplate sink_templ = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("application/x-subtitle; application/x-subtitle-sami; "
-        "application/x-subtitle-tmplayer")
+        "application/x-subtitle-tmplayer; application/x-subtitle-mpl2")
     );
 #else
 static GstStaticPadTemplate sink_templ = GST_STATIC_PAD_TEMPLATE ("sink",
@@ -137,10 +138,6 @@ gst_sub_parse_dispose (GObject * object)
 
   GST_DEBUG_OBJECT (subparse, "cleaning up subtitle parser");
 
-  if (subparse->segment) {
-    gst_segment_free (subparse->segment);
-    subparse->segment = NULL;
-  }
   if (subparse->encoding) {
     g_free (subparse->encoding);
     subparse->encoding = NULL;
@@ -194,14 +191,8 @@ gst_sub_parse_init (GstSubParse * subparse)
   subparse->textbuf = g_string_new (NULL);
   subparse->parser_type = GST_SUB_PARSE_FORMAT_UNKNOWN;
   subparse->flushing = FALSE;
-  subparse->segment = gst_segment_new ();
-  if (subparse->segment) {
-    gst_segment_init (subparse->segment, GST_FORMAT_TIME);
-    subparse->need_segment = TRUE;
-  } else {
-    GST_WARNING_OBJECT (subparse, "segment creation failed");
-    g_assert_not_reached ();
-  }
+  gst_segment_init (&subparse->segment, GST_FORMAT_TIME);
+  subparse->need_segment = TRUE;
   subparse->encoding = g_strdup (DEFAULT_ENCODING);
 }
 
@@ -243,14 +234,11 @@ gst_sub_parse_src_event (GstPad * pad, GstEvent * event)
 
       if (ret) {
         /* Apply the seek to our segment */
-        gst_segment_set_seek (self->segment, rate, format, self->segment_flags,
+        gst_segment_set_seek (&self->segment, rate, format, self->segment_flags,
             start_type, start, stop_type, stop, &update);
 
-        GST_DEBUG_OBJECT (self, "segment configured from %" GST_TIME_FORMAT
-            " to %" GST_TIME_FORMAT ", position %" GST_TIME_FORMAT,
-            GST_TIME_ARGS (self->segment->start),
-            GST_TIME_ARGS (self->segment->stop),
-            GST_TIME_ARGS (self->segment->last_stop));
+        GST_DEBUG_OBJECT (self, "segment after seek: %" GST_SEGMENT_FORMAT,
+            &self->segment);
 
         self->next_offset = 0;
 
@@ -467,10 +455,22 @@ parse_mdvdsub (ParserState * state, const gchar * line)
     if (sscanf (line, "{s:%u}", &fontsize) == 1) {
       line = strchr (line, '}') + 1;
     }
+    /* forward slashes at beginning/end signify italics too */
+    if (g_str_has_prefix (line, "/")) {
+      italic = TRUE;
+      ++line;
+    }
     if ((line_split = strchr (line, '|')))
       line_chunk = g_markup_escape_text (line, line_split - line);
     else
       line_chunk = g_markup_escape_text (line, strlen (line));
+
+    /* Remove italics markers at end of line/stanza (CHECKME: are end slashes
+     * always at the end of a line or can they span multiple lines?) */
+    if (g_str_has_suffix (line_chunk, "/")) {
+      line_chunk[strlen (line_chunk) - 1] = '\0';
+    }
+
     markup = g_string_append (markup, "<span");
     if (italic)
       g_string_append (markup, " style=\"italic\"");
@@ -679,7 +679,8 @@ parse_subrip (ParserState * state, const gchar * line)
       }
       return NULL;
     case 2:
-    {                           /* No need to parse that text if it's out of segment */
+    {
+      /* No need to parse that text if it's out of segment */
       gint64 clip_start = 0, clip_stop = 0;
       gboolean in_seg = FALSE;
 
@@ -696,8 +697,7 @@ parse_subrip (ParserState * state, const gchar * line)
         return NULL;
       }
     }
-      /* looking for subtitle text; empty line ends this
-       * subtitle entry */
+      /* looking for subtitle text; empty line ends this subtitle entry */
       if (state->buf->len)
         g_string_append_c (state->buf, '\n');
       g_string_append (state->buf, line);
@@ -717,6 +717,94 @@ parse_subrip (ParserState * state, const gchar * line)
   }
 }
 
+static void
+subviewer_unescape_newlines (gchar * read)
+{
+  gchar *write = read;
+
+  /* Replace all occurences of '[br]' with a newline as version 2
+   * of the subviewer format uses this for newlines */
+
+  if (read[0] == '\0' || read[1] == '\0' || read[2] == '\0' || read[3] == '\0')
+    return;
+
+  do {
+    if (strncmp (read, "[br]", 4) == 0) {
+      *write = '\n';
+      read += 4;
+    } else {
+      *write = *read;
+      read++;
+    }
+    write++;
+  } while (*read);
+
+  *write = '\0';
+}
+
+static gchar *
+parse_subviewer (ParserState * state, const gchar * line)
+{
+  guint h1, m1, s1, ms1;
+  guint h2, m2, s2, ms2;
+  gchar *ret;
+
+  /* TODO: Maybe also parse the fields in the header, especially DELAY.
+   * For examples see the unit test or
+   * http://www.doom9.org/index.html?/sub.htm */
+
+  switch (state->state) {
+    case 0:
+      /* looking for start_time,end_time */
+      if (sscanf (line, "%u:%u:%u.%u,%u:%u:%u.%u",
+              &h1, &m1, &s1, &ms1, &h2, &m2, &s2, &ms2) == 8) {
+        state->state = 1;
+        state->start_time =
+            (((guint64) h1) * 3600 + m1 * 60 + s1) * GST_SECOND +
+            ms1 * GST_MSECOND;
+        state->duration =
+            (((guint64) h2) * 3600 + m2 * 60 + s2) * GST_SECOND +
+            ms2 * GST_MSECOND - state->start_time;
+      }
+      return NULL;
+    case 1:
+    {
+      /* No need to parse that text if it's out of segment */
+      gint64 clip_start = 0, clip_stop = 0;
+      gboolean in_seg = FALSE;
+
+      /* Check our segment start/stop */
+      in_seg = gst_segment_clip (state->segment, GST_FORMAT_TIME,
+          state->start_time, state->start_time + state->duration,
+          &clip_start, &clip_stop);
+
+      if (in_seg) {
+        state->start_time = clip_start;
+        state->duration = clip_stop - clip_start;
+      } else {
+        state->state = 0;
+        return NULL;
+      }
+    }
+      /* looking for subtitle text; empty line ends this subtitle entry */
+      if (state->buf->len)
+        g_string_append_c (state->buf, '\n');
+      g_string_append (state->buf, line);
+      if (strlen (line) == 0) {
+        ret = g_strdup (state->buf->str);
+        subviewer_unescape_newlines (ret);
+        strip_trailing_newlines (ret);
+        g_string_truncate (state->buf, 0);
+        state->state = 0;
+        return ret;
+      }
+      return NULL;
+    default:
+      g_assert_not_reached ();
+      return NULL;
+  }
+}
+
 static gchar *
 parse_mpsub (ParserState * state, const gchar * line)
 {
@@ -819,7 +907,7 @@ gst_sub_parse_data_format_autodetect (gchar * match_str)
     need_init_regexps = FALSE;
     if ((err = regcomp (&mdvd_rx, "^\\{[0-9]+\\}\\{[0-9]+\\}",
                 REG_EXTENDED | REG_NEWLINE | REG_NOSUB) != 0) ||
-        (err = regcomp (&subrip_rx, "^[1-9]([0-9]){0,3}(\x0d)?\x0a"
+        (err = regcomp (&subrip_rx, "^[0-9]([0-9]){0,3}(\x0d)?\x0a"
                 "[0-9][0-9]:[0-9][0-9]:[0-9][0-9],[0-9]{3}"
                 " --> [0-9][0-9]:[0-9][0-9]:[0-9][0-9],[0-9]{3}",
                 REG_EXTENDED | REG_NEWLINE | REG_NOSUB)) != 0) {
@@ -836,6 +924,7 @@ gst_sub_parse_data_format_autodetect (gchar * match_str)
     GST_LOG ("SubRip (time based) format detected");
     return GST_SUB_PARSE_FORMAT_SUBRIP;
   }
+
   if (!strncmp (match_str, "FORMAT=TIME", 11)) {
     GST_LOG ("MPSub (time based) format detected");
     return GST_SUB_PARSE_FORMAT_MPSUB;
@@ -854,6 +943,14 @@ gst_sub_parse_data_format_autodetect (gchar * match_str)
     GST_LOG ("TMPlayer (time based) format detected");
     return GST_SUB_PARSE_FORMAT_TMPLAYER;
   }
+  if (sscanf (match_str, "[%u][%u]", &n1, &n2) == 2) {
+    GST_LOG ("MPL2 (time based) format detected");
+    return GST_SUB_PARSE_FORMAT_MPL2;
+  }
+  if (strstr (match_str, "[INFORMATION]") != NULL) {
+    GST_LOG ("SubViewer (time based) format detected");
+    return GST_SUB_PARSE_FORMAT_SUBVIEWER;
+  }
 
   GST_DEBUG ("no subtitle format detected");
   return GST_SUB_PARSE_FORMAT_UNKNOWN;
@@ -894,6 +991,12 @@ gst_sub_parse_format_autodetect (GstSubParse * self)
     case GST_SUB_PARSE_FORMAT_TMPLAYER:
       self->parse_line = parse_tmplayer;
       return gst_caps_new_simple ("text/plain", NULL);
+    case GST_SUB_PARSE_FORMAT_MPL2:
+      self->parse_line = parse_mpl2;
+      return gst_caps_new_simple ("text/x-pango-markup", NULL);
+    case GST_SUB_PARSE_FORMAT_SUBVIEWER:
+      self->parse_line = parse_subviewer;
+      return gst_caps_new_simple ("text/plain", NULL);
     case GST_SUB_PARSE_FORMAT_UNKNOWN:
     default:
       GST_DEBUG ("no subtitle format detected");
@@ -944,7 +1047,7 @@ handle_buffer (GstSubParse * self, GstBuffer * buf)
 
   while ((line = get_next_line (self)) && !self->flushing) {
     /* Set segment on our parser state machine */
-    self->state.segment = self->segment;
+    self->state.segment = &self->segment;
     /* Now parse the line, out of segment lines will just return NULL */
     GST_LOG_OBJECT (self, "Parsing line '%s'", line);
     subtitle = self->parse_line (&self->state, line);
@@ -965,7 +1068,7 @@ handle_buffer (GstSubParse * self, GstBuffer * buf)
         GST_BUFFER_TIMESTAMP (buf) = self->state.start_time;
         GST_BUFFER_DURATION (buf) = self->state.duration;
 
-        gst_segment_set_last_stop (self->segment, GST_FORMAT_TIME,
+        gst_segment_set_last_stop (&self->segment, GST_FORMAT_TIME,
             self->state.start_time);
 
         GST_DEBUG_OBJECT (self, "Sending text '%s', %" GST_TIME_FORMAT " + %"
@@ -978,8 +1081,10 @@ handle_buffer (GstSubParse * self, GstBuffer * buf)
       g_free (subtitle);
       subtitle = NULL;
 
-      if (GST_FLOW_IS_FATAL (ret))
+      if (ret != GST_FLOW_OK) {
+        GST_DEBUG_OBJECT (self, "flow: %s", gst_flow_get_name (ret));
         break;
+      }
     }
   }
 
@@ -996,10 +1101,12 @@ gst_sub_parse_chain (GstPad * sinkpad, GstBuffer * buf)
 
   /* Push newsegment if needed */
   if (self->need_segment) {
+    GST_LOG_OBJECT (self, "pushing newsegment event with %" GST_SEGMENT_FORMAT,
+        &self->segment);
+
     gst_pad_push_event (self->srcpad, gst_event_new_new_segment (FALSE,
-            self->segment->rate, self->segment->format,
-            self->segment->last_stop, self->segment->stop,
-            self->segment->time));
+            self->segment.rate, self->segment.format,
+            self->segment.last_stop, self->segment.stop, self->segment.time));
     self->need_segment = FALSE;
   }
 
@@ -1020,7 +1127,8 @@ gst_sub_parse_sink_event (GstPad * pad, GstEvent * event)
     case GST_EVENT_EOS:{
       /* Make sure the last subrip chunk is pushed out even
        * if the file does not have an empty line at the end */
-      if (self->parser_type == GST_SUB_PARSE_FORMAT_SUBRIP) {
+      if (self->parser_type == GST_SUB_PARSE_FORMAT_SUBRIP ||
+          self->parser_type == GST_SUB_PARSE_FORMAT_MPL2) {
         GstBuffer *buf = gst_buffer_new_and_alloc (1 + 1);
 
         GST_DEBUG ("EOS. Pushing remaining text (if any)");
@@ -1040,14 +1148,24 @@ gst_sub_parse_sink_event (GstPad * pad, GstEvent * event)
       gint64 start, stop, time;
       gboolean update;
 
-      GST_DEBUG_OBJECT (self, "received new segment");
-
       gst_event_parse_new_segment (event, &update, &rate, &format, &start,
           &stop, &time);
 
-      /* now copy over the values */
-      gst_segment_set_newsegment (self->segment, update, rate, format,
-          start, stop, time);
+      GST_DEBUG_OBJECT (self, "newsegment (%s)", gst_format_get_name (format));
+
+      if (format == GST_FORMAT_TIME) {
+        gst_segment_set_newsegment (&self->segment, update, rate, format,
+            start, stop, time);
+      } else {
+        /* if not time format, we'll either start with a 0 timestamp anyway or
+         * it's following a seek in which case we'll have saved the requested
+         * seek segment and don't want to overwrite it (remember that on a seek
+         * we always just seek back to the start in BYTES format and just throw
+         * away all text that's before the requested position; if the subtitles
+         * come from an upstream demuxer, it won't be able to handle our BYTES
+         * seek request and instead send us a newsegment from the seek request
+         * it received via its video pads instead, so all is fine then too) */
+      }
 
       ret = TRUE;
       gst_event_unref (event);
@@ -1119,6 +1237,8 @@ gst_sub_parse_change_state (GstElement * element, GstStateChange transition)
 
 /* FIXME 0.11: these caps are ugly, use app/x-subtitle + type field or so;
  * also, give different  subtitle formats really different types */
+static GstStaticCaps mpl2_caps =
+GST_STATIC_CAPS ("application/x-subtitle-mpl2");
 static GstStaticCaps tmp_caps =
 GST_STATIC_CAPS ("application/x-subtitle-tmplayer");
 static GstStaticCaps smi_caps = GST_STATIC_CAPS ("application/x-subtitle-sami");
@@ -1127,6 +1247,7 @@ static GstStaticCaps sub_caps = GST_STATIC_CAPS ("application/x-subtitle");
 #define SUB_CAPS (gst_static_caps_get (&sub_caps))
 #define SAMI_CAPS (gst_static_caps_get (&smi_caps))
 #define TMP_CAPS (gst_static_caps_get (&tmp_caps))
+#define MPL2_CAPS (gst_static_caps_get (&mpl2_caps))
 
 static void
 gst_subparse_type_find (GstTypeFind * tf, gpointer private)
@@ -1165,6 +1286,18 @@ gst_subparse_type_find (GstTypeFind * tf, gpointer private)
       GST_DEBUG ("TMPlayer (time based) format detected");
       caps = TMP_CAPS;
       break;
+      /* FIXME: our MPL2 typefinding is not really good enough to warrant
+       * returning a high probability (however, since we registered our
+       * typefinder here with a rank of MARGINAL we should pretty much only
+       * be called if most other typefinders have already run */
+    case GST_SUB_PARSE_FORMAT_MPL2:
+      GST_DEBUG ("MPL2 (time based) format detected");
+      caps = MPL2_CAPS;
+      break;
+    case GST_SUB_PARSE_FORMAT_SUBVIEWER:
+      GST_DEBUG ("SubViewer format detected");
+      caps = SUB_CAPS;
+      break;
     default:
     case GST_SUB_PARSE_FORMAT_UNKNOWN:
       GST_DEBUG ("no subtitle format detected");
