@@ -31,10 +31,10 @@ G_BEGIN_DECLS
 #define GST_VIDEO_TEST_SRC(obj) \
   (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_VIDEO_TEST_SRC,GstVideoTestSrc))
 #define GST_VIDEO_TEST_SRC_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_VIDEO_TEST_SRC,GstVideoTestSrc))
+  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_VIDEO_TEST_SRC,GstVideoTestSrcClass))
 #define GST_IS_VIDEO_TEST_SRC(obj) \
   (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_VIDEO_TEST_SRC))
-#define GST_IS_VIDEO_TEST_SRC_CLASS(obj) \
+#define GST_IS_VIDEO_TEST_SRC_CLASS(klass) \
   (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_VIDEO_TEST_SRC))
 
 /**
@@ -42,6 +42,16 @@ G_BEGIN_DECLS
  * @GST_VIDEO_TEST_SRC_SMPTE: A standard SMPTE test pattern
  * @GST_VIDEO_TEST_SRC_SNOW: Random noise
  * @GST_VIDEO_TEST_SRC_BLACK: A black image
+ * @GST_VIDEO_TEST_SRC_WHITE: A white image
+ * @GST_VIDEO_TEST_SRC_RED: A red image
+ * @GST_VIDEO_TEST_SRC_GREEN: A green image
+ * @GST_VIDEO_TEST_SRC_BLUE: A blue image
+ * @GST_VIDEO_TEST_SRC_CHECKERS1: Checkers pattern (1px)
+ * @GST_VIDEO_TEST_SRC_CHECKERS2: Checkers pattern (2px)
+ * @GST_VIDEO_TEST_SRC_CHECKERS4: Checkers pattern (4px)
+ * @GST_VIDEO_TEST_SRC_CHECKERS8: Checkers pattern (8px)
+ * @GST_VIDEO_TEST_SRC_CIRCULAR: Circular pattern
+ * @GST_VIDEO_TEST_SRC_BLINK: Alternate between black and white
  *
  * The test pattern to produce.
  */
@@ -49,6 +59,16 @@ typedef enum {
   GST_VIDEO_TEST_SRC_SMPTE,
   GST_VIDEO_TEST_SRC_SNOW,
   GST_VIDEO_TEST_SRC_BLACK,
+  GST_VIDEO_TEST_SRC_WHITE,
+  GST_VIDEO_TEST_SRC_RED,
+  GST_VIDEO_TEST_SRC_GREEN,
+  GST_VIDEO_TEST_SRC_BLUE,
+  GST_VIDEO_TEST_SRC_CHECKERS1,
+  GST_VIDEO_TEST_SRC_CHECKERS2,
+  GST_VIDEO_TEST_SRC_CHECKERS4,
+  GST_VIDEO_TEST_SRC_CHECKERS8,
+  GST_VIDEO_TEST_SRC_CIRCULAR,
+  GST_VIDEO_TEST_SRC_BLINK
 } GstVideoTestSrcPattern;
 
 typedef struct _GstVideoTestSrc GstVideoTestSrc;
@@ -62,6 +82,8 @@ typedef struct _GstVideoTestSrcClass GstVideoTestSrcClass;
 struct _GstVideoTestSrc {
   GstPushSrc element;
 
+  /*< private >*/
+
   /* type of output */
   GstVideoTestSrcPattern pattern_type;
 
